#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Mon March  4 2024

@author: C. Junger, S. Schoder
"""

import numpy as np
import matplotlib.pyplot as plt

from matplotlib import rc
from cycler import cycler 

rc('text', usetex=True)
plt.rc('font', family='serif')
rc('font',size=16.0)
rc('lines',linewidth=1.5) 
rc('legend',fontsize='medium',framealpha=1.0,numpoints=1) 
rc('svg',fonttype='none')
color_cycle = cycler('color',['#377eb8','#4daf4a','#e41a1c','#984ea3','#ff7f00','#a65628','#f781bf','#999999','#ffff33'])
rc('axes',prop_cycle=color_cycle,grid=True,linewidth=1.0,axisbelow=True,unicode_minus=False)

def delta(l) :
    return 0.05*Re**-0.2 * l

def G(St) :
    return 0.01 / (1+4.1985*St+0.454*St**6)

def S_p(f) :
    St = f*delta(l)/w
    return rho**2 * w**3 * delta(l) * G(St)

def S_bl(b,l,w,f) :
    y = []
    for freq in f :
        x = np.pi*freq*l/w
        if x <= 2 :
            y.append((b*l**2 *w / (5*np.pi*freq)) *S_p(freq))
        elif 2 < x <= 15/np.pi :
            y.append(((2*b* l * w**2) / (5*np.pi**2 * freq**2)) *S_p(freq))
        else :
            y.append((6*b*w**3 / (np.pi**4 * freq**3)) *S_p(freq))
    npa = np.asarray(y, dtype=np.float64)
    return npa

def F(f, Lambda,c) :
    a = [-9.784, -19.001, -5.548, -0.060]
    F = 0
    for k in range(0,4) :
        F = F + a[k]*np.log10(f*Lambda/c)**(k)
    return F
    
def S_w(f,Lambda,c) :
    return c * Tu**2 * Lambda * 10**(F(f,Lambda,c)/10)
 
def pegel(x) :
    # conversion in Power spectral density:
    # L_W = L_p + 10*np.log10(A1/A0) + 10*np.log10(p0**2*A0/(rho*c0*P0))
    return x+10*np.log10(2*np.pi*1**2)+10*np.log10(2e-5**2*1/(rho*c0*P0))


# fan characteristics
vol_flowrate = 1.4 # volumetric flow
D = 0.5 # Laufraddurchmesser
n = 1486.0 # rotational speed 1/min
u = np.pi*D*n/60 # rotational velocity m/s
c0 = 343.0 # speed of sound
z = 9.0 # number of blades
r_i = 0.248/2 # hub radius
r_o = 0.495/2 # tip radius
l = (0.103 + 0.058)/2 # mean chord length
b = r_o - r_i # chord width
Psi = 0.9*np.pi # gradient of lift coefficient
rho = 1.225 # density
w = np.pi*(r_i+b/2)*n/30 # relative velocity in the middle of the blade
c = vol_flowrate /(np.pi *D**2/4) # mean velocity
nu = r_i / r_o # nabenverhaeltniss
#c=w

Re = 1.5*10**5 # reynolds number
psi = 1.0 # radial function

# from measurement
dpts = 128.0 # total-to-static pressure difference
eta = 0.53 # total-to-static efficiency from measurement
Lambda = 0.051 # turbulent length scale in x direction
Tu = 0.07 # turbulent intensity 0.05 ueber weite bereiche

f = np.arange(100.0,10000.0,1.0)

S_ti = z * 0.81 * np.pi / 48.0 * rho/c0**3 * w**4 * S_w(f,Lambda,c) *l *b
S_tbl = z * (np.pi/4) * f/(rho* c0**2 * r_o*(1-nu**2)**2) * S_bl(b,l,w,f) * psi

P0 = 10**-12

P_ti = S_ti * (f[1]-f[0]) 
#L_ti = 10*np.log10(P_ti/P0)
P_tbl = S_tbl * (f[1]-f[0])
#L_tbl = 10*np.log10(P_tbl/P0) 

S = S_ti+S_tbl  
P = S * (f[1]-f[0]) 
L = 10*np.log10(P/P0)

# read measurement data
meas = "./measurement_df16hz_mic4.csv"
dsm = np.loadtxt(meas,delimiter=';',skiprows=1)
band = "./measurement_df16hz_0p1s_mic4.csv"
dsb = np.loadtxt(band,delimiter=';',skiprows=1)
rows, columns = dsb.shape
# get max values of band
maxs = []
mins = []
fextrs = np.copy(dsb[:,2])
for k in range(0,columns,2):
    dsb[:,k] = None  
for i in range(0,rows):
    maxs = np.append(maxs, np.nanmax(dsb[i,:]))
    mins = np.append(mins, np.nanmin(dsb[i,:]))

fig = plt.figure()
ax = fig.add_subplot(111)
ax.set_xlabel('$f$ / Hz')
ax.set_ylabel('PSD / (dB/Hz)')
ax.set_ylim([-20,80])
ax.set_xlim([100,10000])
# plot measurement
ax.semilogx(dsm[:,0],dsm[:,1],'black',label='Measurement (30.0\,s)')
ax.fill_between(fextrs,maxs,mins,color='grey', label="Measurement (0.1\,s)")
ax.semilogx(f,L,label='Költzsch')
ax.grid(True,which='both')
ax.legend()
plt.show()
fig.savefig('PSD_compare.pdf',bbox_inches='tight')
